"""Separate text to zh en lists."""
# pylint: disable=unused-import, too-many-locals, invalid-name, too-many-branches, too-many-statements,


# from typing import Tuple,
from typing import Iterable, List, Optional, Tuple, Union  # noqa

import numpy as np

# from radiobee.lists2cmat import lists2cmat  # use fast_scores
# from radiobee.detect import detect
from fast_scores.gen_cmat import gen_cmat  # pylint: disable=import-error
from logzero import logger

# from fastlid import fastlid
from polyglot.text import Detector

from ezbee.detect import detect


def text2lists(
    text: Union[Iterable[str], str],
    set_languages: Optional[List[str]] = None,
) -> Tuple[List[str], List[str]]:
    """Separate text to zh en lists.

    Args:
        text: mixed text
        set_languages: no default (open-end)
            use polyglot.text.Detector to pick two languages

    Attributes:
        cmat: correlation matrix (len(list_l) x len(list_r))
            before adjusting (shifting)
        offset: plus, [""] * offset + list2
                minus, [""] * (-offset) + list1
    Returns:
        two lists, best effort alignment
    """
    if not isinstance(text, str) and isinstance(text, Iterable):
        try:
            text = "\n".join(text)
        except Exception as e:
            logger.error(e)
            raise

    # set_languages default to ["en", "zh"]
    if set_languages is None:
        lang12 = [elm.code for elm in Detector(text).languages]

        # set_languages = ["en", "zh"]

        # set 'un' to 'en'
        # set_languages = ['en' if elm in ['un'] else elm for elm in lang12[:2]]
        set_languages = []
        for elm in lang12[:2]:
            if elm in ["un"]:
                logger.warning(" Unknown language, set to en")
                set_languages.append("en")
            else:
                set_languages.append(elm)

    # fastlid.set_languages = set_languages

    list1 = []
    list2 = []

    # lang0, _ = fastlid(text[:15000])
    lang0 = detect(text, set_languages)

    res = []
    left = True  # start with left list1

    for elm in [_ for _ in text.splitlines() if _.strip()]:
        # lang, _ = fastlid(elm)
        lang = detect(elm, set_languages)
        if lang == lang0:
            res.append(elm)
        else:
            if left:
                # list1.append("\n".join(res))
                list1.extend(res)
            else:
                # list2.append("\n".join(res))
                list2.extend(res)
            left = not left

            res = [elm]
            lang0 = lang

    # process the last
    if left:
        list1.extend(res)
    else:
        list2.extend(res)

    try:
        # lang1, _ = fastlid(' '.join(list1))
        lang1 = detect(" ".join(list1), set_languages)
    except Exception as exc:
        logger.error(exc)
        lang1 = "en"
    try:
        # lang2, _ = fastlid(' '.join(list2))
        lang2 = detect(" ".join(list2), set_languages)
    except Exception as exc:
        logger.error(exc)
        lang2 = "en"

    # find offset via diagonal(k),
    len1, len2 = len(list1), len(list2)

    # ylim, xlim = cmat.shape
    ylim, xlim = len2, len1  # check

    # cmat dim: len1 x len2 or ylim x xlim
    # cmat = lists2cmat(list1, list2, lang1, lang2)
    # cmat.shape: len(list1)xlen(list2) or ylim x xlim
    cmat = gen_cmat(list1, list2)

    # sq_mean_pair = [(elm, np.square(cmat.diagonal(elm)).mean()) for elm in range(2 - ylim, xlim + 1)]
    # df = pd.DataFrame(sq_mean_pair, columns=['offset', 'sq_mean'])
    # df.plot.scatter('offset', 'sq_mean')
    # optimum_offset = df.offset[df.sq_mean.argmax()]

    # equiv to np.argmax(sq_mean) - (ylim - 2)
    # locate max, -ylim + 2 ...xlim: range(1 - ylim, xlim)
    # sqare sum

    sq_mean = [
        np.square(cmat.diagonal(elm)).mean() for elm in range(1 - ylim, xlim - 1)
    ]
    # tot: xlim + ylim - 1

    # temp = [np.square(cmat.diagonal(elm)) for elm in range(2 - ylim, xlim + 1)]
    # sq_mean = [elm.mean() if np.any(elm) else 0.0 for elm in temp]

    # plt.figure()
    # plt.scatter(range(1 - ylim, xlim), sq_mean)

    offset = np.argmax(sq_mean) - (ylim - 1)

    text2lists.cmat = cmat
    text2lists.offset = offset
    text2lists.lang1 = lang1
    text2lists.lang2 = lang2

    # shift list1 if offsset >= 0, else shift list2
    if offset > -1:
        # list1a = list1[:]
        # list2a = [""] * offset + list2
        list2 = [""] * offset + list2
    else:
        list1 = [""] * (-offset) + list1
        # list1a = [""] * (-offset) + list1
        # list2a = list2[:]

    # return list1, list2
    return [elm.strip() for elm in list1], [elm.strip() for elm in list2]



"""Define ezbee."""
# pylint: disable=invalid-name
from typing import List

from cmat2aset import cmat2aset  # pylint: disable=import-error
from fast_scores.gen_cmat import gen_cmat  # pylint: disable=import-error
from logzero import logger


def ezbee(
    text1: List[str],
    text2: List[str],
    eps: float = 10,
    min_samples: int = 6,
    # show_plot: bool = True,
) -> List:
    """Align en-zh/zh-en texts.

    Args:
        text1: list of strings
        text2: list of strings
        eps: epsilon
        min_samples: minimum number of points to be considered as a cluster
        x show_plot: whether to show plots, refactored to cmat2html's show_plot
    Returns:
        Aligned text pairs.
    """
    if not ([elm for elm in text1 if elm.strip()] and [elm for elm in text2 if elm.strip()]):
        logger.warning("One or both inputs are empty")
        raise Exception("Nothing to do...exiting")

    try:
        cmat = gen_cmat(text1, text2)
        # logger.level is reset to 20 in fastlid
        ezbee.cmat = cmat
    except Exception as e:
        logger.exception(e)
        raise

    try:
        # aset = cmat2aset(cmat.T)
        aset = cmat2aset(cmat, eps=eps, min_samples=min_samples)
        ezbee.aset = aset
    except Exception as e:
        logger.exception(e)
        raise

    # paired texts or aset?
    return aset



"""Prep __main__ entry."""
# pylint: disable=invalid-name, too-many-locals, too-many-arguments, too-many-branches, too-many-statements, duplicate-code
from pathlib import Path
from textwrap import dedent
from typing import List, Optional

# import environs
import logzero
import typer
from icecream import ic, install as ic_install
from logzero import logger

from ezbee import __version__, ezbee, loglevel

# from ezbee.cmat2html import cmat2html
from ezbee.loadtext import loadtext
from ezbee.text2lists import text2lists
from ezbee.gen_pairs import gen_pairs
from ezbee.save_xlsx_tsv_csv import save_xlsx_tsv_csv

# logzero.loglevel(_)
logzero.loglevel(loglevel())


# logger.info(" loglevel: %s", _ or 20)

# logger.debug(" debug: %s", __file__)
# logger.info(" info: %s", __file__)

ic_install()
ic.configureOutput(
    includeContext=True,
    # outputFunction=logger.info,
    outputFunction=logger.debug,
)
ic.enable()

app = typer.Typer(
    name="ezbee",
    add_completion=False,
    help="en-zh-bee aligner",
)

esp_min_samples_expl = dedent(
    """
    Larger esp or smaller min_samples will result in more aligned pairs but also more false positives (pairs falsely identified as candidates). On the other hand, smaller esp or larger min_samples values tend to miss `good` pairs."""
).strip()


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(
            f"{app.info.name} v.{__version__} -- visit https://bumblebee.freeforums.net/thread/2/ezbee-cli-related to chat about {app.info.name}."
        )
        raise typer.Exit()


@app.command()
def main(
    files: List[str] = typer.Argument(
        ...,
        metavar="file1 [file2]...",
        help="files (absolute or relative paths) to be aligned; if only one file is specified, the -s flag must be used to signal it's an english/chinese mixed text file and needs to be separated.",
    ),
    eps: float = typer.Option(10, help="epsilon"),
    min_samples: int = typer.Option(
        6, help="eps, min-samples: " + esp_min_samples_expl
    ),
    need_sep: bool = typer.Option(
        False,
        "--need-sep",
        "-s",
        is_flag=True,
        help="Separate input files that are mixed english and chinese text.",
    ),
    show_plot: bool = typer.Option(
        False,
        "--show-plot",
        "-p",
        is_flag=True,
        help="Show heatmap and align trace plots in the default browser.",
    ),
    save_xlsx: bool = typer.Option(
        True,
        help="Save xlsx.",
    ),
    save_tsv: bool = typer.Option(
        True,
        help="Save tsv.",
    ),
    save_csv: bool = typer.Option(
        False,
        help="Save csv.",
    ),
    version: Optional[bool] = typer.Option(  # pylint: disable=(unused-argument
        None,
        "--version",
        "-v",
        "-V",
        help="Show version info and exit.",
        callback=_version_callback,
        is_eager=True,
    ),
):
    """Align en-zh texts.

    e.g.

    * ezbee file1 file2

    * ezbee file1 file2 -p  # show plots

    * ezbee file1 -s

    * ezbee file1 file2 -s
    """
    logger.info("Collecting inputs...")
    # test show-plot -p
    # logger.info(show_plot)
    logger.debug("show-plot: %s", show_plot)

    logger.debug("files: %s", files)

    if not show_plot and not save_xlsx and not save_tsv and not save_csv:
        exiting = typer.style("exiting...", fg=typer.colors.RED, bold=True)
        typer.echo(
            "None of show-plot, save-xlsx, save-tsv and save-csv is set to True: nothing to do, "
            f"{exiting}"
        )
        raise typer.Exit(code=0)

    if not files:
        typer.echo("Provide at least one file")
        raise typer.Exit()

    if len(files) == 1 and not need_sep:
        typer.echo(
            "If you only provide one file, you'll also have to specify -s or --need-sep."
        )
        typer.echo("Try again...")
        raise typer.Exit()

    for file_ in files:
        if not Path(file_).is_file():
            typer.echo(f" [{file_}] is not a file or does not exist.")
            raise typer.Exit()

    # paired, two files and need_sep not set
    if len(files) == 2 and not need_sep:
        try:
            text1 = loadtext(files[0])
        except Exception as e:
            logger.error(e)
            raise
        try:
            text2 = loadtext(files[1])
        except Exception as e:
            logger.error(e)
            raise

        list1 = [elm.strip() for elm in text1.splitlines() if elm.strip()]
        list2 = [elm.strip() for elm in text2.splitlines() if elm.strip()]

        logger.debug("len1: %s, len2: %s", len(list1), len(list2))

    # other cases: 2 files + need_sep not set, 3 or or files
    else:  # assume mixed english/chinese, separate
        list1 = []
        list2 = []
        text = ""
        for file in files:
            text = text + "\n" + loadtext(file)
        list1, list2 = text2lists(text)

    # text1 = Path(files[0]).read_text(encoding="utf8").splitlines()
    # text2 = Path(files[1]).read_text(encoding="utf8").splitlines()

    # logger.debug("list1: %s", list1)
    # logger.debug("list2: %s", list2)

    # typer.echo(list1)
    # typer.echo(list2)
    # raise typer.Exit(1)

    try:
        logger.info("Processing data...")
        aset = ezbee(
            list1,
            list2,
            eps=eps,
            min_samples=min_samples,
        )
        # fastlid changed logger.level is changed to 20
        # turn back to loglevel
        logzero.loglevel(loglevel())
        if aset:
            logger.debug("aset: %s...%s", aset[:3], aset[-3:])
    except Exception as e:
        typer.echo(e)
        raise typer.Exit()

    # process show_plot
    logger.debug("show-plot: %s", show_plot)
    if show_plot:
        try:
            from ezbee.cmat2html import cmat2html  # pylint: disable=import-outside-toplevel
            flag = False
        except ModuleNotFoundError:
            logger.warning(" You need to install extras [plot] in order to use this option, e.g. pip install ezbee[plot]==0.1.0a3, exiting...")

            # raise typer.Exit(1) from exc
            flag = True
        except Exception as exc:
            logger.exception(exc)
            flag = True

        if not flag:
            cmat2html(
                ezbee.cmat,
                aset=ezbee.aset,
                show_plot=show_plot,
            )

    logger.debug("Proceed with saving files...")

    aligned_pairs = gen_pairs(list1, list2, aset)
    if aligned_pairs:
        logger.debug("%s...%s", aligned_pairs[:3], aligned_pairs[-3:])

    _ = [save_xlsx, save_tsv, save_csv]
    file_ext = [v1 for v0, v1 in zip(_, [".xlsx", ".tsv", ".csv"]) if v0]
    logger.debug("[save_xlsx, save_tsv, save_csv]: %s, file_ext: %s", _, file_ext)

    if not file_ext:  # nothing to do
        raise typer.Exit(code=0)
    # file =

    logger.info("Saving %s", file_ext)

    _ = Path(files[0]).with_suffix("").as_posix() + "-ali"
    _ = Path(_)
    save_xlsx_tsv_csv(
        aligned_pairs,
        file_ext=file_ext,
        file=_,
    )

    raise typer.Exit(code=0)


if __name__ == "__main__":
    app()


_ = """
https://medium.com/short-bits/typer-the-command-line-script-framework-you-should-use-de9d07109f54
one of many:

from enum import Enum
class FileExt(Enum):
    csv = ".csv"
    tsv = ".tsv"
    xlsx = ".xlsx"

def __main__(
    ...
    file_exc: FileExt = FileExt.xlsx
    ...
)
    typer.echo(f" file_ext: {file_exc.value}")
"""



"""Set loglevel."""
# pylint: disable=duplicate-code
import environs


def loglevel(
    level: int = 20,
    force: bool = False,
) -> int:
    """Return an integer based on env LOGLEVEL.

    Args:
        level: set loglevel if env LOGLEVEL is not set.
        force: bool, use level if set.
        if force is not set, env LOGLEVEL takes precedence.
        set env LOGLEVEL to 10/debug/DEBUG to turn on debug
    Returns:
        an integer for using in logzero.loglevel()
        if env LOGLEVEL is not set, use level.

    >>> loglevel(force=True)
    20
    >>> import os; os.environ['logleve'] = 'debug'
    >>> loglevel()
    10
    """
    try:
        level = int(level)
    except Exception:
        level = 20

    if force:
        return level

    try:
        _ = environs.Env().log_level("LOGLEVEL")
    except (environs.EnvError, environs.EnvValidationError):
        _ = None
    except Exception:
        _ = None

    _ = _ or level
    return _



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ezbee']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.1,<4.0.0',
 'Morfessor>=2.0.6,<3.0.0',
 'XlsxWriter>=3.0.3,<4.0.0',
 'cchardet>=2.1.7,<3.0.0',
 'cmat2aset==0.1.0a7',
 'environs>=9.5.0,<10.0.0',
 'fast-scores==0.1.3a3',
 'icecream>=2.1.1,<3.0.0',
 'install>=1.3.5,<2.0.0',
 'logzero>=1.7.0,<2.0.0',
 'polyglot>=16.7.4,<17.0.0',
 'typer>=0.4.0,<0.5.0']

extras_require = \
{'plot': ['holoviews>=1.14.8,<2.0.0',
          'plotly>=5.6.0,<6.0.0',
          'seaborn>=0.11.2,<0.12.0']}

entry_points = \
{'console_scripts': ['ezbee = ezbee.__main__:app']}

setup_kwargs = {
    'name': 'ezbee',
    'version': '0.1.1a0',
    'description': 'engnlish-chinese dualtext aligner',
    'long_description': '# ezbee\n[![pytest](https://github.com/ffreemt/ezbee/actions/workflows/windows-pytest.yml/badge.svg)](https://github.com/ffreemt/ezbee/actions)[![python](https://img.shields.io/static/v1?label=python+&message=3.8%2B&color=blue)](https://www.python.org/downloads/)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/ezbee.svg)](https://badge.fury.io/py/ezbee)\n\nezbee to align en/zh texts\n\n## Install it\n\n```shell\npip install git+https://github.com/ffreemt/ezbee\n# poetry add git+https://github.com/ffreemt/ezbee\n# git clone https://github.com/ffreemt/ezbee && cd ezbee\n```\n\n## Use it\n```python\nezbee --help\n# or python -m ezbee --help\n```\n',
    'author': 'ffreemt',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ffreemt/ezbee',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8.3,<4.0.0',
}


setup(**setup_kwargs)



"""Load file content to text.

Check encoding and load a file to text.

Win
Linux
    apt install libmagic1

py -3.8 -m pip install python-magic-bin
py -3.8 -m pip install python-magic

import magic
magic.from_file("testdata/test.pdf")

original load_textrev
refer to load_paras.py
"""
# pylint: disable=invalid-name, line-too-long, unused-variable, unused-import

import io
from pathlib import Path
from typing import Union

import cchardet

# import pytest
from logzero import logger

# from detect_file import detect_file


def loadtext(filepath: Union[io.BytesIO, Path, str] = "") -> str:
    """Load file context to text.

    Check encoding and load a file to text.
    """
    # check for file-like object
    if isinstance(filepath, io.BytesIO):
        try:
            readable = filepath.readable()
        except AttributeError:
            readable = False

        if readable:
            filepath.seek(0)
            encoding, conf = cchardet.detect(filepath.read()).values()

            if encoding is None:
                logger.error("Unable to determine encoding, exiting...")
                raise SystemExit(1)

            try:
                filepath.seek(0)
                return filepath.read().decode(encoding)
            except Exception as e:
                logger.error(e)
                raise

    # proceed with other options
    try:
        filepath = Path(filepath)  # type: ignore
    except Exception as e:
        logger.error(e)
        raise

    if not filepath.is_file():
        logger.error(" file [%s] does not exist or is not a file.", filepath)
        # return None
        raise Exception(f" file [{filepath}] does not exist or is not a file.")

    # encoding = detect_file(filepath)
    encoding = cchardet.detect(filepath.read_bytes()).get("encoding", "utf8")

    if encoding is None:
        raise Exception("cchardet.detect says it's not a text file.")

    # cchardet: 'GB18030', no need for errors="ignore"
    try:
        text = filepath.read_text(encoding=encoding, errors="ignore")
    except Exception as exc:
        logger.error(" Opening %s resulted in errors: %s", filepath, exc)
        raise

    # return text
    return text.replace("\u3000", " ")


#
